@@ -22,7 +22,6 @@
 #'   }
 #'   This parameter allows fine-grained control over the noise characteristics applied to each motif (mandatory).
 #' @param seed_background An integer value setting the seed for the random number generator used in background curve generation. This ensures reproducibility of the background curves. Default is `777`.
-#' @param seed_motif An integer value setting the seed for the random number generator used in motif generation. This ensures reproducibility of the motif embedding process. Default is `43213`.
 #' @param only_der A logical value indicating whether to apply only derivative-based modifications to the motifs (`TRUE`) or to add a vertical shift in addition to derivative modifications (`FALSE`). Setting to `FALSE` introduces vertical shifts to each motif instance, allowing motifs to appear at different baseline levels within the curves. Default is `TRUE`.
 #' @param coeff_min_shift A numeric value specifying the minimum vertical shift to be applied to motifs when `only_der` is set to `FALSE`. This parameter controls the lower bound of the vertical displacement of motifs. Default is `-10`.
 #' @param coeff_max_shift A numeric value specifying the maximum vertical shift to be applied to motifs when `only_der` is set to `FALSE`. This parameter controls the upper bound of the vertical displacement of motifs. Default is `10`.
@@ -119,8 +118,8 @@
 #' # Generate curves with coefficient noise and vertical shifts
 #' curves <- funMoDisco::generateCurves(builder, noise_type = 'coeff', noise_str, only_der = FALSE)
 #' }
-setGeneric("generateCurves", function(object,noise_type = NULL, noise_str = NULL,seed_background = 777,seed_motif = 43213,
-                                      only_der = TRUE,coeff_min_shift = -10,coeff_max_shift = 10) {
+setGeneric("generateCurves", function(object,noise_type = NULL, noise_str = NULL,seed_background = NULL,
+                                      only_der = FALSE, coeff_min_shift = -10,coeff_max_shift = 10) {
   standardGeneric("generateCurves")
 })
 
@@ -149,10 +148,9 @@ setGeneric("generateCurves", function(object,noise_type = NULL, noise_str = NULL
 #'   }
 #'   This parameter allows fine-grained control over the noise characteristics applied to each motif (mandatory).
 #' @param seed_background An integer value setting the seed for the random number generator used in background curve generation. This ensures reproducibility of the background curves. Default is `777`.
-#' @param seed_motif An integer value setting the seed for the random number generator used in motif generation. This ensures reproducibility of the motif embedding process. Default is `43213`.
-#' @param only_der A logical value indicating whether to apply only derivative-based modifications to the motifs (`TRUE`) or to add a vertical shift in addition to derivative modifications (`FALSE`). Setting to `FALSE` introduces vertical shifts to each motif instance, allowing motifs to appear at different baseline levels within the curves. Default is `TRUE`.
-#' @param coeff_min_shift A numeric value specifying the minimum vertical shift to be applied to motifs when `only_der` is set to `FALSE`. This parameter controls the lower bound of the vertical displacement of motifs. Default is `-10`.
-#' @param coeff_max_shift A numeric value specifying the maximum vertical shift to be applied to motifs when `only_der` is set to `FALSE`. This parameter controls the upper bound of the vertical displacement of motifs. Default is `10`.
+#' @param only_der A logical value indicating whether to apply only derivative-based modifications to the motifs (`TRUE`) or to add a vertical shift in addition to derivative modifications (`FALSE`). Setting to `TRUE` introduces vertical shifts to each motif instance, allowing motifs to appear at different baseline levels within the curves. Default is `FALSE`.
+#' @param coeff_min_shift A numeric value specifying the minimum vertical shift to be applied to motifs when `only_der` is set to `TRUE`. This parameter controls the lower bound of the vertical displacement of motifs. Default is `-10`.
+#' @param coeff_max_shift A numeric value specifying the maximum vertical shift to be applied to motifs when `only_der` is set to `TRUE`. This parameter controls the upper bound of the vertical displacement of motifs. Default is `10`.
 #'
 #' @return 
 #' A list containing the following components:
@@ -164,6 +162,9 @@ setGeneric("generateCurves", function(object,noise_type = NULL, noise_str = NULL
 #'   \item \strong{SNR}: A list of Signal-to-Noise Ratio (SNR) metrics calculated for each motif within each curve, useful for assessing the quality of motif embedding.
 #' }
 #'
+#' @export 
+#' 
+#' @importFrom purrr is_empty
 #' @examples
 #' \donttest{
 #' # Example 0: Special case with no motifs
@@ -246,12 +247,21 @@ setGeneric("generateCurves", function(object,noise_type = NULL, noise_str = NULL
 #' # Generate curves with coefficient noise and vertical shifts
 #' curves <- funMoDisco::generateCurves(builder, noise_type = 'coeff', noise_str, only_der = FALSE)
 #' }
-setMethod("generateCurves", "motifSimulation", function(object,noise_type = NULL, noise_str = NULL,seed_background = 777,seed_motif = 43213,
-                                                        only_der = TRUE,coeff_min_shift = -10,coeff_max_shift = 10) {
-  breaks=lapply(rep(object@len,length.out=object@N),seq,from=0,by=object@dist_knots) # generate N lists with equally spaced nodes 'dist_knots' from 0 to 'len'
-  basis=lapply(breaks,function(breaks_i) create.bspline.basis(norder=object@norder,breaks=breaks_i)) # nbasis = norder + nbreaks - 2 = norder + interior_knots,
-  len_motifs <- unlist(lapply(object@mot_details,function(x){x$len}))
+setMethod("generateCurves", "motifSimulation", function(object,
+                                                        noise_type = NULL,
+                                                        noise_str = NULL,
+                                                        seed_background = NULL,
+                                                        only_der = FALSE,
+                                                        coeff_min_shift = -10,
+                                                        coeff_max_shift = 10){
+  
+  breaks = lapply(rep(object@len, length.out = object@N), seq, from = 0, by = object@dist_knots) # generate N lists with equally spaced nodes 'dist_knots' from 0 to 'len'
+  basis = lapply(breaks, function(breaks_i) create.bspline.basis(norder = object@norder, breaks = breaks_i)) # nbasis = norder + nbreaks - 2 = norder + interior_knots,
+  len_motifs <- unlist(lapply(object@mot_details, function(x){x$len}))
+  
+  # if the seed is user defined (not null) then set the seed
   set.seed(seed_background)
+  
   # loop for each curve
   fd_curves <- NULL
   if(is.null(noise_type) && !is.null(noise_str)) {
@@ -261,52 +271,56 @@ setMethod("generateCurves", "motifSimulation", function(object,noise_type = NULL
     stop("\'noise_type'\ must be specified because \'noise_str\' is not null")
   }
   if((is.null(noise_type) && !is_empty(object@motifs_in_curves))) {
-    stop("\'noise_type\' must be choosen between \'coeff\' and \'pointiwse\'")
+    stop("\'noise_type\' must be choosen between \'coeff\' and \'pointwise\'")
   } 
   if(is.null(noise_type)) {
-    noise_type <- "pointwise" # to generate background curves without noise
+    noise_type <- "coeff" # to generate background curves without noise
   }
+  
   if(noise_type == 'coeff') {
+    
     tryCatch({
       # Attempt to coerce the input to a numeric vector
-      noise_str <- lapply(noise_str,function(error){return(as.numeric(error))})
+      noise_str <- lapply(noise_str, function(error){return(as.numeric(error))})
     }, error = function(e) {
       stop("\'noise_str'\ cannot be vectorized.")
     })
+    
     if(length(noise_str) != length(object@mot_details)) {
-     stop("\'noise_str\' must have the same length of \'mot_details\'") 
+      stop("\'noise_str\' must have the same length of \'mot_details\'") 
     }
+    
     fd_curves <- mapply(function(motifs_in_curves_i,len_i,basis_i,j,len_motifs){
       list_coeff_error <- NULL
       if(is.numeric(object@distribution)) {
-          or_coeff=sample(object@distribution,size = len_i,replace = TRUE)
-      } else if(object@distribution=='unif'){
-          or_coeff=runif(len_i,min=object@coeff_min,max=object@coeff_max) # coefficients of the curve = degree of freedom are initialized uniformly
-      } else if(object@distribution=='beta') { 
-          or_coeff=object@coeff_min+rbeta(len_i,0.45,0.45)*(object@coeff_max-object@coeff_min) # coefficients of the curve = degree of freedom are initialized uniformly
+        or_coeff = sample(object@distribution,size = len_i,replace = TRUE)
+      } else if(object@distribution == 'unif'){
+        or_coeff = runif(len_i, min = object@coeff_min, max = object@coeff_max) # coefficients of the curve = degree of freedom are initialized uniformly
+      } else if(object@distribution == 'beta') { 
+        or_coeff = object@coeff_min + rbeta(len_i,0.45,0.45)*(object@coeff_max - object@coeff_min) # coefficients of the curve = degree of freedom are initialized uniformly
       } else {
         stop('Wrong \'distrib\'')
       }
       coeff <- or_coeff
-      fda_no_error=fda::fd(coef=coeff,basisobj=basis_i)
+      fda_no_error = fda::fd(coef = coeff, basisobj = basis_i)
       or_y_no_error <- generate_curve_vector(fda_no_error)
       fda_with_error <- NULL
       or_y <- NULL
       shifted_coeff <- NULL
-      set.seed(seed_motif)
+      #set.seed(seed_motif)
       if(!is.null(motifs_in_curves_i)) {
-        pos_coeff_motifs=unlist(mapply(
-          function(a,b) seq(a)+b,
+        pos_coeff_motifs = unlist(mapply(
+          function(a,b) seq(a) + b,
           rep(len_motifs,length.out=length(object@mot_details))[motifs_in_curves_i$motif_id]/object@dist_knots+object@norder-1, # number of motifs coefficients for each selected motif
-          motifs_in_curves_i$starting_coeff_pos-1,SIMPLIFY=FALSE)) # Calculating the position of the coefficients of the motifs within the coefficients of the curves
-        shifted_coeff <- rep(runif(length(motifs_in_curves_i$motif_id),min=coeff_min_shift,max=coeff_max_shift),rep(len_motifs,length.out=length(object@mot_details))[motifs_in_curves_i$motif_id]/object@dist_knots+object@norder-1) 
-      
+          motifs_in_curves_i$starting_coeff_pos - 1,SIMPLIFY=FALSE)) # Calculating the position of the coefficients of the motifs within the coefficients of the curves
+        shifted_coeff <- rep(runif(length(motifs_in_curves_i$motif_id), min = coeff_min_shift, max = coeff_max_shift),rep(len_motifs,length.out=length(object@mot_details))[motifs_in_curves_i$motif_id]/object@dist_knots+object@norder-1) 
+        
         list_coeff_error <- lapply(1:length(noise_str[[1]]),function(z) {
-          if(only_der) {
-          coeff[pos_coeff_motifs] = unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
-            sd_noise <- noise_str[[id]][z]
-            message(paste("\033[34m --- Adding motif", id, "to curve", j, "with noise", sd_noise, "\033[0m"))
-            object@mot_details[[id]]$coeffs + rnorm(length(object@mot_details[[id]]$coeffs),sd=sd_noise)}))
+          if(only_der == FALSE) {
+            coeff[pos_coeff_motifs] = unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
+              sd_noise <- noise_str[[id]][z]
+              message(paste("\033[34m --- Adding motif", id, "to curve", j, "with noise", sd_noise, "\033[0m"))
+              object@mot_details[[id]]$coeffs + rnorm(length(object@mot_details[[id]]$coeffs),sd=sd_noise)}))
           } else { 
             coeff[pos_coeff_motifs] = unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
               sd_noise <- noise_str[[id]][z]
@@ -316,69 +330,72 @@ setMethod("generateCurves", "motifSimulation", function(object,noise_type = NULL
           # For each chosen coefficient add a uniform number and a gaussian noise
           return(coeff)
         })
-      if(only_der) {
-        coeff[pos_coeff_motifs] <- unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
-                                  object@mot_details[[id]]$coeffs}))
-      } else {
-        coeff[pos_coeff_motifs] <- unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
-                                    object@mot_details[[id]]$coeffs})) + shifted_coeff
-      }
-      fda_with_motif <- fda::fd(coef=coeff,basisobj=basis_i)
-      or_y_motif <- generate_curve_vector(fda_with_motif)
-      
-      fda_with_error <- Map(fda::fd,list_coeff_error,MoreArgs = list(basisobj=basis_i)) # Fitting curves using such coefficients and basis
-      or_y <- Map(generate_curve_vector,fda_with_error)
-      num_motifs <- length(motifs_in_curves_i$motif_id)
-      SNR <- vector("list",length(noise_str[[1]]))
-      for (k in seq_along(noise_str[[1]])) {
-        SNR_num <- data.frame(xmin = numeric(), xmax = numeric(), SNR = numeric(), stringsAsFactors = FALSE)
-        SNR_den <- data.frame(xmin = numeric(), xmax = numeric(), SNR = numeric(), stringsAsFactors = FALSE)
-      
-        # Loop over each motif
-        for (n in seq_along(motifs_in_curves_i$motif_id)) {
-          start_break <- motifs_in_curves_i$starting_coeff_pos[n]
-          start_point <- (start_break - 1) * object@dist_knots
-          end_point   <- start_point + len_motifs[motifs_in_curves_i$motif_id[n]]
-
-          # Calculate variance for SNR numerator and denominator
-          SNR_num <- rbind(SNR_num,data.frame(xmin =start_point,xmax = end_point,SNR = var(or_y_motif[start_point:end_point])))
-          SNR_den <- rbind(SNR_den,data.frame(xmin =start_point,xmax = end_point,SNR = var(or_y[[k]][start_point:end_point] - or_y_motif[start_point:end_point])))
+        if(only_der == FALSE) {
+          coeff[pos_coeff_motifs] <- unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
+            object@mot_details[[id]]$coeffs}))
+        } else {
+          coeff[pos_coeff_motifs] <- unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
+            object@mot_details[[id]]$coeffs})) + shifted_coeff
         }
-        # Calculate SNR in decibels for the k-th error
-        SNR[[k]] <- SNR_num
-        SNR[[k]]$SNR <- 10 * log10(SNR_num$SNR / SNR_den$SNR) #transform SNR in decibel
-      }
-      return(list(basis = basis_i,
-                  background = list(or_coeff = or_coeff,no_error_y = or_y_no_error),
-                  no_noise = list(or_coeff = coeff,motif_y = or_y_motif),
-                  with_noise = list(noise_structure = noise_str,noise_y = or_y),
-                  SNR = SNR))
-      }
+        fda_with_motif <- fda::fd(coef = coeff, basisobj = basis_i)
+        or_y_motif <- generate_curve_vector(fda_with_motif)
         
+        fda_with_error <- Map(fda::fd,list_coeff_error,MoreArgs = list(basisobj=basis_i)) # Fitting curves using such coefficients and basis
+        or_y <- Map(generate_curve_vector,fda_with_error)
+        num_motifs <- length(motifs_in_curves_i$motif_id)
+        SNR <- vector("list",length(noise_str[[1]]))
+        for (k in seq_along(noise_str[[1]])) {
+          SNR_num <- data.frame(xmin = numeric(), xmax = numeric(), SNR = numeric(), stringsAsFactors = FALSE)
+          SNR_den <- data.frame(xmin = numeric(), xmax = numeric(), SNR = numeric(), stringsAsFactors = FALSE)
+          
+          # Loop over each motif
+          for (n in seq_along(motifs_in_curves_i$motif_id)) {
+            start_break <- motifs_in_curves_i$starting_coeff_pos[n]
+            start_point <- (start_break - 1) * object@dist_knots
+            end_point   <- start_point + len_motifs[motifs_in_curves_i$motif_id[n]]
+            
+            # Calculate variance for SNR numerator and denominator
+            SNR_num <- rbind(SNR_num,data.frame(xmin =start_point,xmax = end_point,SNR = var(or_y_motif[start_point:end_point])))
+            SNR_den <- rbind(SNR_den,data.frame(xmin =start_point,xmax = end_point,SNR = var(or_y[[k]][start_point:end_point] - or_y_motif[start_point:end_point])))
+          }
+          # Calculate SNR in decibels for the k-th error
+          SNR[[k]] <- SNR_num
+          SNR[[k]]$SNR <- 10 * log10(SNR_num$SNR / SNR_den$SNR) #transform SNR in decibel
+        }
+        return(list(basis = basis_i,
+                    background = list(or_coeff = or_coeff,no_error_y = or_y_no_error),
+                    no_noise = list(or_coeff = coeff,motif_y = or_y_motif),
+                    with_noise = list(noise_structure = noise_str,noise_y = or_y),
+                    SNR = SNR))
+      }
+      
       return(list(basis = basis_i,
                   background = list(or_coeff = or_coeff,no_error_y = or_y_no_error)))
     },object@motifs_in_curves,object@norder-1+rep(object@len/object@dist_knots,length.out=object@N),basis,1:object@N,MoreArgs = list(len_motifs),SIMPLIFY=FALSE)
     
   }else if(noise_type == 'pointwise'){
+    
     if(length(noise_str) != length(object@mot_details)) {
       stop("\'noise_str\' must have the same length of \'mot_details\'") 
     }
+    
     fd_curves <- lapply(1:object@N, function(x){
       coeff <- NULL
-      len_i <- object@norder-1+object@len/object@dist_knots
+      len_i <- object@norder - 1 + object@len/object@dist_knots
       if(is.numeric(object@distribution)) {
-        coeff=sample(object@distribution,size = len_i,replace = TRUE)
+        coeff = sample(object@distribution, size = len_i, replace = TRUE)
       }
       else if(object@distribution=='unif'){
-        coeff=runif(len_i,min=object@coeff_min,max=object@coeff_max) # If we don't have motifs we sample len_i coefficients uniformely
+        coeff = runif(len_i, min = object@coeff_min, max = object@coeff_max) # If we don't have motifs we sample len_i coefficients uniformely
       }else if(object@distribution=='beta'){
-        coeff=object@coeff_min+rbeta(len_i,0.45,0.45)*(object@coeff_max-object@coeff_min) 
+        coeff = object@coeff_min + rbeta(len_i, 0.45, 0.45)*(object@coeff_max - object@coeff_min) 
       }else {
         stop('Wrong \'distrib\'')
       }
-      generate_background_curve(object@len, object@dist_knots, object@norder,coeff, add_noise = TRUE)
+      generate_background_curve(object@len, object@dist_knots, object@norder, coeff, add_noise = TRUE)
     })
-    set.seed(seed_motif)
+    
+    #set.seed(seed_motif)
     curve_ids <- unique(unlist(lapply(object@mot_details,function(mot){mot$occurrences$curve})))
     for(j in curve_ids){
       message(paste("\033[34m --- Adding motifs to curve", j, "\033[0m"))
@@ -408,6 +425,175 @@ setMethod("generateCurves", "motifSimulation", function(object,noise_type = NULL
   return(fd_curves = fd_curves)
 })
 
+# PREVIOUS VERSION
+# setMethod("generateCurves", "motifSimulation", function(object, noise_type = NULL, noise_str = NULL, seed_background = NULL, seed_motif = NULL,
+#                                                         only_der = FALSE, coeff_min_shift = -10, coeff_max_shift = 10) {
+#   breaks = lapply(rep(object@len, length.out = object@N), seq, from = 0, by = object@dist_knots) # generate N lists with equally spaced nodes 'dist_knots' from 0 to 'len'
+#   basis = lapply(breaks, function(breaks_i) create.bspline.basis(norder = object@norder, breaks = breaks_i)) # nbasis = norder + nbreaks - 2 = norder + interior_knots,
+#   len_motifs <- unlist(lapply(object@mot_details, function(x){x$len}))
+#   
+#   # if the seed is user defined (not null) then set the seed
+#   if(!is.null(seed_background)){
+#     set.seed(seed_background)
+#   }
+# 
+#   # loop for each curve
+#   fd_curves <- NULL
+#   if(is.null(noise_type) && !is.null(noise_str)) {
+#     stop("\'noise_str'\ must be specified because \'noise_type\' is not null")
+#   }
+#   if(is.null(noise_str) && !is.null(noise_type)) {
+#     stop("\'noise_type'\ must be specified because \'noise_str\' is not null")
+#   }
+#   if((is.null(noise_type) && !is_empty(object@motifs_in_curves))) {
+#     stop("\'noise_type\' must be choosen between \'coeff\' and \'pointiwse\'")
+#   } 
+#   if(is.null(noise_type)) {
+#     noise_type <- "pointwise" # to generate background curves without noise
+#   }
+#   if(noise_type == 'coeff') {
+#     tryCatch({
+#       # Attempt to coerce the input to a numeric vector
+#       noise_str <- lapply(noise_str, function(error){return(as.numeric(error))})
+#     }, error = function(e) {
+#       stop("\'noise_str'\ cannot be vectorized.")
+#     })
+#     if(length(noise_str) != length(object@mot_details)) {
+#      stop("\'noise_str\' must have the same length of \'mot_details\'") 
+#     }
+#     fd_curves <- mapply(function(motifs_in_curves_i,len_i,basis_i,j,len_motifs){
+#       list_coeff_error <- NULL
+#       if(is.numeric(object@distribution)) {
+#           or_coeff = sample(object@distribution,size = len_i,replace = TRUE)
+#       } else if(object@distribution == 'unif'){
+#           or_coeff = runif(len_i, min = object@coeff_min, max = object@coeff_max) # coefficients of the curve = degree of freedom are initialized uniformly
+#       } else if(object@distribution == 'beta') { 
+#           or_coeff = object@coeff_min + rbeta(len_i,0.45,0.45)*(object@coeff_max - object@coeff_min) # coefficients of the curve = degree of freedom are initialized uniformly
+#       } else {
+#         stop('Wrong \'distrib\'')
+#       }
+#       coeff <- or_coeff
+#       fda_no_error = fda::fd(coef = coeff, basisobj = basis_i)
+#       or_y_no_error <- generate_curve_vector(fda_no_error)
+#       fda_with_error <- NULL
+#       or_y <- NULL
+#       shifted_coeff <- NULL
+#       #set.seed(seed_motif)
+#       if(!is.null(motifs_in_curves_i)) {
+#         pos_coeff_motifs = unlist(mapply(
+#           function(a,b) seq(a) + b,
+#           rep(len_motifs,length.out=length(object@mot_details))[motifs_in_curves_i$motif_id]/object@dist_knots+object@norder-1, # number of motifs coefficients for each selected motif
+#           motifs_in_curves_i$starting_coeff_pos - 1,SIMPLIFY=FALSE)) # Calculating the position of the coefficients of the motifs within the coefficients of the curves
+#         shifted_coeff <- rep(runif(length(motifs_in_curves_i$motif_id), min = coeff_min_shift, max = coeff_max_shift),rep(len_motifs,length.out=length(object@mot_details))[motifs_in_curves_i$motif_id]/object@dist_knots+object@norder-1) 
+#       
+#         list_coeff_error <- lapply(1:length(noise_str[[1]]),function(z) {
+#           if(only_der == FALSE) {
+#           coeff[pos_coeff_motifs] = unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
+#             sd_noise <- noise_str[[id]][z]
+#             message(paste("\033[34m --- Adding motif", id, "to curve", j, "with noise", sd_noise, "\033[0m"))
+#             object@mot_details[[id]]$coeffs + rnorm(length(object@mot_details[[id]]$coeffs),sd=sd_noise)}))
+#           } else { 
+#             coeff[pos_coeff_motifs] = unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
+#               sd_noise <- noise_str[[id]][z]
+#               message(paste("\033[34m --- Adding motif", id, "to curve", j, "with noise", sd_noise, "\033[0m"))
+#               object@mot_details[[id]]$coeffs + rnorm(length(object@mot_details[[id]]$coeffs),sd=sd_noise)})) + shifted_coeff
+#           }
+#           # For each chosen coefficient add a uniform number and a gaussian noise
+#           return(coeff)
+#         })
+#       if(only_der == FALSE) {
+#         coeff[pos_coeff_motifs] <- unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
+#                                   object@mot_details[[id]]$coeffs}))
+#       } else {
+#         coeff[pos_coeff_motifs] <- unlist(lapply(motifs_in_curves_i$motif_id, function(id) {
+#                                     object@mot_details[[id]]$coeffs})) + shifted_coeff
+#       }
+#       fda_with_motif <- fda::fd(coef = coeff, basisobj = basis_i)
+#       or_y_motif <- generate_curve_vector(fda_with_motif)
+#       
+#       fda_with_error <- Map(fda::fd,list_coeff_error,MoreArgs = list(basisobj=basis_i)) # Fitting curves using such coefficients and basis
+#       or_y <- Map(generate_curve_vector,fda_with_error)
+#       num_motifs <- length(motifs_in_curves_i$motif_id)
+#       SNR <- vector("list",length(noise_str[[1]]))
+#       for (k in seq_along(noise_str[[1]])) {
+#         SNR_num <- data.frame(xmin = numeric(), xmax = numeric(), SNR = numeric(), stringsAsFactors = FALSE)
+#         SNR_den <- data.frame(xmin = numeric(), xmax = numeric(), SNR = numeric(), stringsAsFactors = FALSE)
+#       
+#         # Loop over each motif
+#         for (n in seq_along(motifs_in_curves_i$motif_id)) {
+#           start_break <- motifs_in_curves_i$starting_coeff_pos[n]
+#           start_point <- (start_break - 1) * object@dist_knots
+#           end_point   <- start_point + len_motifs[motifs_in_curves_i$motif_id[n]]
+# 
+#           # Calculate variance for SNR numerator and denominator
+#           SNR_num <- rbind(SNR_num,data.frame(xmin =start_point,xmax = end_point,SNR = var(or_y_motif[start_point:end_point])))
+#           SNR_den <- rbind(SNR_den,data.frame(xmin =start_point,xmax = end_point,SNR = var(or_y[[k]][start_point:end_point] - or_y_motif[start_point:end_point])))
+#         }
+#         # Calculate SNR in decibels for the k-th error
+#         SNR[[k]] <- SNR_num
+#         SNR[[k]]$SNR <- 10 * log10(SNR_num$SNR / SNR_den$SNR) #transform SNR in decibel
+#       }
+#       return(list(basis = basis_i,
+#                   background = list(or_coeff = or_coeff,no_error_y = or_y_no_error),
+#                   no_noise = list(or_coeff = coeff,motif_y = or_y_motif),
+#                   with_noise = list(noise_structure = noise_str,noise_y = or_y),
+#                   SNR = SNR))
+#       }
+#         
+#       return(list(basis = basis_i,
+#                   background = list(or_coeff = or_coeff,no_error_y = or_y_no_error)))
+#     },object@motifs_in_curves,object@norder-1+rep(object@len/object@dist_knots,length.out=object@N),basis,1:object@N,MoreArgs = list(len_motifs),SIMPLIFY=FALSE)
+#     
+#   }else if(noise_type == 'pointwise'){
+#     if(length(noise_str) != length(object@mot_details)) {
+#       stop("\'noise_str\' must have the same length of \'mot_details\'") 
+#     }
+#     fd_curves <- lapply(1:object@N, function(x){
+#       coeff <- NULL
+#       len_i <- object@norder - 1 + object@len/object@dist_knots
+#       if(is.numeric(object@distribution)) {
+#         coeff = sample(object@distribution, size = len_i, replace = TRUE)
+#       }
+#       else if(object@distribution=='unif'){
+#         coeff = runif(len_i, min = object@coeff_min, max = object@coeff_max) # If we don't have motifs we sample len_i coefficients uniformely
+#       }else if(object@distribution=='beta'){
+#         coeff = object@coeff_min + rbeta(len_i, 0.45, 0.45)*(object@coeff_max - object@coeff_min) 
+#       }else {
+#         stop('Wrong \'distrib\'')
+#       }
+#       generate_background_curve(object@len, object@dist_knots, object@norder, coeff, add_noise = TRUE)
+#     })
+#     
+#     #set.seed(seed_motif)
+#     curve_ids <- unique(unlist(lapply(object@mot_details,function(mot){mot$occurrences$curve})))
+#     for(j in curve_ids){
+#       message(paste("\033[34m --- Adding motifs to curve", j, "\033[0m"))
+#       temp_curve <- fd_curves[[j]]
+#       temp_pattern <- do.call(rbind,lapply(object@mot_details,function(mot_details){(mot_details$occurrences %>% filter(curve == j)) %>% dplyr::select(motif_id, start_break_pos)}))
+#       temp_len <- do.call(rbind,lapply(unique(apply(temp_pattern,1,function(row){row[1]})),function(k){data.frame("motif_id" = k,"len" = object@mot_details[[k]]$len)}))
+#       temp_weights <- lapply(as.character(unique(apply(temp_pattern, 1, function(row) { row[1] }))), 
+#                              function(k) { 
+#                                object@mot_details[[as.numeric(k)]]$coeffs
+#                              })
+#       names(temp_weights) <- as.character(unique(apply(temp_pattern, 1, function(row) { row[1] })))
+#       fd_curves[[j]] <- add_motif(base_curve  = temp_curve,
+#                                   mot_pattern = temp_pattern,
+#                                   mot_len     = temp_len,
+#                                   dist_knots  = object@dist_knots,
+#                                   mot_order   = object@norder,
+#                                   mot_weights = temp_weights,
+#                                   noise_str   = noise_str,
+#                                   only_der = only_der,
+#                                   coeff_min_shift = coeff_min_shift,
+#                                   coeff_max_shift = coeff_max_shift)
+#     }
+#     fd_curves <- .transform_list(fd_curves,noise_str)
+#   } else {
+#     stop("\'noise_type\' must be choosen between \'coeff\' and \'pointwise\'")
+#   }
+#   return(fd_curves = fd_curves)
+# })
+
 #' @title Plot Embedded Motifs in Functional Curves
 #'
 #' @description
@@ -426,19 +612,24 @@ setMethod("generateCurves", "motifSimulation", function(object,noise_type = NULL
 #' The function does not return any value but generates and saves plots of the functional curves with embedded motifs in the specified directory. Each plot visually represents the motifs within the curves, aiding in the qualitative assessment of motif embedding.
 #'
 #' @export
+#' 
+#' @importFrom stats setNames
+#' @importFrom ggtext element_markdown
 #'
 #' @examples
 #' \donttest{
 #' # Example: Plotting motifs in generated curves
 #' # Assume 'builder' has been created and 'curves' have been generated using generateCurves
-#' builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details)
-#' curves <- funMoDisco::generateCurves(builder, noise_type = 'pointwise', noise_str = noise_str)
+#' 
+#' mot_details <- NULL
+#' builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details = mot_details)
+#' curves <- funMoDisco::generateCurves(builder)
 #'
 #' # Specify the directory to save plots
 #' plots_name <- "plots_1"  
 #'
 #' # Generate and save the plots
-#' funMoDisco::plot_motifs(builder, curves, plots_name)
+#' funMoDisco::plot_motifs(builder, curves, plots_name, path = tempdir())
 #' }
 setGeneric("plot_motifs", function(object,curves,name,path) 
   standardGeneric("plot_motifs")
@@ -467,17 +658,18 @@ setGeneric("plot_motifs", function(object,curves,name,path)
 #' \donttest{
 #' # Example: Plotting motifs in generated curves
 #' # Assume 'builder' has been created and 'curves' have been generated using generateCurves
+#' mot_details <- NULL
 #' builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details)
-#' curves <- funMoDisco::generateCurves(builder, noise_type = 'pointwise', noise_str = noise_str)
+#' curves <- funMoDisco::generateCurves(builder)
 #'
 #' # Specify the directory to save plots
 #' plots_name <- "plots_1"  
 #'
 #' # Generate and save the plots
-#' funMoDisco::plot_motifs(builder, curves, plots_name)
+#' funMoDisco::plot_motifs(builder, curves, plots_name, path = tempdir())
 #' }
 setMethod("plot_motifs","motifSimulation",
-          function(object,curves,name,path) {
+          function(object, curves, name, path) {
             
   # Ensure the path is absolute
   if (!grepl("^(/|[A-Za-z]:\\\\)", path)) {  
@@ -517,7 +709,7 @@ setMethod("plot_motifs","motifSimulation",
         
         # Title
         labs(
-          title = paste0('<b><span style="color:#0073C2;">Random curve ', k, '</span></b>'),
+          title = paste0('<b><span style="color:#0073C2;"> Curve ', k, '</span></b>'),
           x = "t",
           y = "x"
         ) +
@@ -563,7 +755,7 @@ setMethod("plot_motifs","motifSimulation",
                          "5" = "purple", "6" = "cyan", "7" = "magenta", "8" = "brown",
                          "9" = "pink", "10" = "grey")
       motif_colors <- rep(motif_colors,length.out = length(object@mot_details))
-      if(length(object@mot_details) > 10 )
+      if( length(object@mot_details) > 10 )
         attr(motif_colors,"names")[11:length(object@mot_details)] <- as.character(as.integer(attr(motif_colors,"names")[11:length(object@mot_details)]) + 10)
   
       max_dataframes <- max(sapply(motif_lines, function(sublist) length(sublist)))
@@ -583,7 +775,7 @@ setMethod("plot_motifs","motifSimulation",
           # Plot the main curve in gray30
           geom_line(data = curve_data_no_error, aes(x = t, y = x, color = 'background_curve'), linewidth = 0.5) +
           # Plot the curve with motifs in gold
-          geom_line(data = curve_data_no_error, aes(x = t, y = z, color = 'zero_noise_motif'), linewidth = 0.5) +
+          geom_line(data = curve_data_no_error, aes(x = t, y = z, color = 'zero_noise_curve'), linewidth = 0.5) +
           # Plot the error curve
           geom_line(data = curve_data_error, aes_string(x = "t", y = paste0("x", j)), color = "black", linewidth = 0.5) +
           # Add shaded rectangles for motif positions
@@ -600,10 +792,10 @@ setMethod("plot_motifs","motifSimulation",
           # Add color and fill scales with custom labels for motif_id
           scale_color_manual(
             values = c('background_curve' = scales::alpha('gray30', 0.15), 
-                       'zero_noise_motif' = 'gold', 
+                       'zero_noise_curve' = 'gold', 
                        motif_colors),
             labels = c('background_curve' = 'background_curve', 
-                       'zero_noise_motif' = 'zero_noise_motif', 
+                       'zero_noise_curve' = 'zero_noise_curve', 
                        setNames(motif_labels, unique(motif_data[[j]]$initial_number)))
           ) +
           scale_fill_manual(
@@ -612,7 +804,7 @@ setMethod("plot_motifs","motifSimulation",
           ) +
           # Title
           labs(
-            title = paste0('<b><span style="color:#0073C2;">Random curve ', k, ' - type_error ', 
+            title = paste0('<b><span style="color:#0073C2;">Curve ', k, ' - type_error ', 
                            ifelse(j %% length(curves[[k]]$with_noise$noise_y) == 0, 
                                   length(curves[[k]]$with_noise$noise_y), j %% length(curves[[k]]$with_noise$noise_y)), 
                            '</span></b>'), 
@@ -694,17 +886,23 @@ setMethod("plot_motifs","motifSimulation",
           if(motif_in_curve_i$motif_id[z] == id) {
             start <- (motif_in_curve_i$starting_coeff_pos[z] - 1) * object@dist_knots + 1
             # Compute pointwise means across all curves in the subcurve list
-            if (is.null(motif_y_means[[index]])) {
-              motif_y_means[[index]] <- curve$no_noise$motif_y[start:(start + object@mot_details[[index]]$len - 1)]
-            } else {
-              motif_y_means[[index]] <- motif_y_means[[index]] + curve$no_noise$motif_y[start:(start + object@mot_details[[index]]$len - 1)]
-            }
+            
+            motif_y_means[[index]] <- curve$no_noise$motif_y[start:(start + object@mot_details[[index]]$len - 1)]
+            
+            # MODIFIED BY ME: this code was computing the mean of the noisy occurrences
+            # if (is.null(motif_y_means[[index]])) {
+            #   motif_y_means[[index]] <- curve$no_noise$motif_y[start:(start + object@mot_details[[index]]$len - 1)]
+            # } else {
+            #   motif_y_means[[index]] <- motif_y_means[[index]] + curve$no_noise$motif_y[start:(start + object@mot_details[[index]]$len - 1)]
+            # }
           }
         }
       }
-      motif_y_means[[index]] <- data.frame(x = seq_along(motif_y_means[[index]]), y = motif_y_means[[index]] / length(object@mot_details[[index]]$occurrences$curve %>% unique()))
+      #motif_y_means[[index]] <- data.frame(x = seq_along(motif_y_means[[index]]), y = motif_y_means[[index]] / length(object@mot_details[[index]]$occurrences$curve %>% unique()))
+      motif_y_means[[index]] <- data.frame(x = seq_along(motif_y_means[[index]]), y = motif_y_means[[index]] %>% unique())
       names(motif_y_means)[index] <- id
     }
+    
     # Loop per plottare ogni ID separatamente su pagine diverse
     for (id in unique_ids) {
       for(error_n in 1:max_dataframes) {
@@ -723,7 +921,7 @@ setMethod("plot_motifs","motifSimulation",
             subtitle = paste0("Mean of the no noise motif <b>", id, "</b> with type error <b>", error_n, "</b>"),
             x = "x",
             y = "t",
-            color = "Noise Curves"
+            color = "Mot. occurrences"
           ) + 
           theme_minimal(base_size = 15) + 
           theme(
@@ -739,7 +937,7 @@ setMethod("plot_motifs","motifSimulation",
             panel.grid.minor = element_blank()
           ) + 
           guides(
-            color = guide_legend(ncol = 1, byrow = TRUE, title = "Noise Curves"),
+            color = guide_legend(ncol = 1, byrow = TRUE, title = "Mot. occurrences"),
             fill = "none"
           )
         print(p)
@@ -756,7 +954,12 @@ setMethod("plot_motifs","motifSimulation",
 #' @export
 #' @examples
 #' \donttest{
-#' curves <- funMoDisco::generateCurves(builder, noise_type = 'coeff', noise_str, only_der = FALSE)
+#' N <- 20 # number of curves
+#' len <- 300 # length of the curves
+#' mot_details <- NULL # or list()
+#'
+#' builder <- motifSimulationBuilder(N = N, len = len, mot_details = NULL)
+#' curves <- funMoDisco::generateCurves(builder)
 #' formatted_curves <- to_motifDiscovery(curves)
 #' }
 setGeneric("to_motifDiscovery", function(curves) 
@@ -770,7 +973,8 @@ setGeneric("to_motifDiscovery", function(curves)
 #' @export
 #' @examples
 #' \donttest{
-#' curves <- funMoDisco::generateCurves(builder, noise_type = 'coeff', noise_str, only_der = FALSE)
+#' builder <- funMoDisco::motifSimulationBuilder(N = 20, len = 300, mot_details = NULL)
+#' curves <- funMoDisco::generateCurves(builder)
 #' formatted_curves <- to_motifDiscovery(curves)
 #' }
 setMethod("to_motifDiscovery",c(curves = "list"), function(curves) {
