@@ -69,7 +69,7 @@
 #' It also supports the visualization of cleaned motifs, which are motifs that have been refined based on specific criteria to ensure quality and relevance.
 #'
 #' @export
-probKMA_plot <- function(probKMA_results,plot,ylab='',sil_avg=NULL,cleaned=FALSE, transformed = FALSE){
+probKMA_plot <- function(probKMA_results, plot, ylab='',sil_avg=NULL,cleaned = FALSE, transformed = FALSE){
   d=ncol(probKMA_results$Y0[[1]])
   N=nrow(probKMA_results$P)
   K=ncol(probKMA_results$P)
@@ -85,299 +85,309 @@ probKMA_plot <- function(probKMA_results,plot,ylab='',sil_avg=NULL,cleaned=FALSE
     P_clean_i=split(probKMA_results$P_clean,seq_len(nrow(probKMA_results$S_clean)))
     has_a_motif <- sapply(P_clean_i,function(p_i){sum(p_i)!=0}) # at least one motif embedded
     if(is.null(probKMA_results$V1[[1]])){
-      #### Plot curves all together with the embedded motifs############################################################################################
-      mapply(function(v0,v_dom,s_k,p_clean_k,k)
-        {
-        layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
-        keep=which(p_clean_k==1)
-        # matrix extension (eventually)
-        full_mat <- probKMA_results$Y0
-        # max len of the curves
-        maxLen <-  max(sapply(full_mat, nrow))
-        # Extended full matrix
-        full_mat <- sapply(full_mat, padding, maxLen,simplify = FALSE)
-        # Join
-        full_mat <- do.call(cbind, full_mat)
-        dom_sequence <- sapply(s_k[keep], function(x){seq(x,x+max(0,length(v_dom)-1))},simplify=TRUE)
-        # Extract keep submatrix
-        column <- numeric(length(keep) * d)
-        for (i in 1:length(keep)) {
-          start_col <- (keep[i] - 1) * d + 1
-          end_col <- start_col + d - 1
-          column[((i - 1) * d + 1):((i - 1) * d + d)] <- start_col:end_col
-        }
-        # Plot curves all together 
-        lapply(seq_len(d),
-               function(j){
-                 par(mar=c(3,4,4,2)+0.1)
-                 matplot(full_mat[,seq(j,ncol(full_mat),by = d)],col=scales::alpha('gray30',0.30),lwd=1.5,lty=1,type = 'l',
-                         main=paste0('Curves & ','Motif_',k,' - Dimension:',j,'\n',
-                                     'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
-                                     ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))))
-                 univariate_mat <- as.matrix(as.matrix(full_mat[,column])[,seq(j,d*length(keep),by = d)])
-                 matplot(univariate_mat,type="l",col=rainbow(length(keep)),lwd=1.5,lty=5,
-                         main=paste0('Curves & ','Motif_',k,' - Dimension:',j,'\n',
-                                     'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
-                                     ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))),add=TRUE)
-                 for (i in 1:ncol(dom_sequence)) {
-                   lines(dom_sequence[,i],univariate_mat[dom_sequence[,i],i], col = 'red',lwd=2.5)
-                   rect(dom_sequence[1,i], min(univariate_mat,na.rm=TRUE)-10, tail(dom_sequence[,i], n=1), max(univariate_mat,na.rm=TRUE)+10,
-                        border = scales::alpha(rainbow(length(keep))[i], 0.05), col = scales::alpha(rainbow(length(keep))[i], 0.05))
-                 }
-                 title(sub = paste('Curves & ', 'Motif:', k, ' - Dimension:', j))
-                 par(mar=c(0,0,0,0))
-                 plot.new()
-                 legend("topright", legend = c(paste0("c",(1:N)[keep]),paste0('motif_',k)),
-                        col = c(rainbow(length(keep)),'red'), lty = 1,lwd=c(rep(1,length(keep)),4),cex=1.5)
-               })
-        return()},probKMA_results$V0_clean,V_dom,S_clean_k,P_clean_k,seq_len(K))
-    
+      NULL
+      #### Plot curves all together with the embedded motifs -------------
+      # colorful plot (deleted)
+      # mapply(function(v0,v_dom,s_k,p_clean_k,k)
+      #   {
+      #   layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
+      #   keep=which(p_clean_k==1)
+      #   # matrix extension (eventually)
+      #   full_mat <- probKMA_results$Y0
+      #   # max len of the curves
+      #   maxLen <-  max(sapply(full_mat, nrow))
+      #   # Extended full matrix
+      #   full_mat <- sapply(full_mat, padding, maxLen,simplify = FALSE)
+      #   # Join
+      #   full_mat <- do.call(cbind, full_mat)
+      #   dom_sequence <- sapply(s_k[keep], function(x){seq(x,x+max(0,length(v_dom)-1))},simplify=TRUE)
+      #   # Extract keep submatrix
+      #   column <- numeric(length(keep) * d)
+      #   for (i in 1:length(keep)) {
+      #     start_col <- (keep[i] - 1) * d + 1
+      #     end_col <- start_col + d - 1
+      #     column[((i - 1) * d + 1):((i - 1) * d + d)] <- start_col:end_col
+      #   }
+      #   # Plot curves all together 
+      #   lapply(seq_len(d),
+      #          function(j){
+      #            par(mar=c(3,4,4,2)+0.1)
+      #            matplot(full_mat[,seq(j,ncol(full_mat),by = d)],col=scales::alpha('gray30',0.30),lwd=1.5,lty=1,type = 'l',
+      #                    main=paste0('Curves & ','Motif_',k,' - Dimension:',j,'\n',
+      #                                'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
+      #                                ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))))
+      #            univariate_mat <- as.matrix(as.matrix(full_mat[,column])[,seq(j,d*length(keep),by = d)])
+      #            matplot(univariate_mat,type="l",col=rainbow(length(keep)),lwd=1.5,lty=5,
+      #                    main=paste0('Curves & ','Motif_',k,' - Dimension:',j,'\n',
+      #                                'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
+      #                                ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))),add=TRUE)
+      #            for (i in 1:ncol(dom_sequence)) {
+      #              lines(dom_sequence[,i],univariate_mat[dom_sequence[,i],i], col = 'red',lwd=2.5)
+      #              rect(dom_sequence[1,i], min(univariate_mat,na.rm=TRUE)-10, tail(dom_sequence[,i], n=1), max(univariate_mat,na.rm=TRUE)+10,
+      #                   border = scales::alpha(rainbow(length(keep))[i], 0.05), col = scales::alpha(rainbow(length(keep))[i], 0.05))
+      #            }
+      #            title(sub = paste('Curves & ', 'Motif:', k, ' - Dimension:', j))
+      #            par(mar=c(0,0,0,0))
+      #            plot.new()
+      #            legend("topright", legend = c(paste0("c",(1:N)[keep]),paste0('motif_',k)),
+      #                   col = c(rainbow(length(keep)),'red'), lty = 1,lwd=c(rep(1,length(keep)),4),cex=1.5)
+      #          })
+      #   return()},probKMA_results$V0_clean,V_dom,S_clean_k,P_clean_k,seq_len(K))
+      # 
       #### for each curve plot the embedded motifs  ############################################################################################
-      mapply(function(curve_i,s_i,p_clean_i,i){
-        layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
-        lapply(seq_len(d),
-               function(j)
-                 {
-                  par(mar=c(3,4,4,2)+0.1)
-                  univariate_mat <- curve_i[,j]
-                  matplot(univariate_mat,type="l",col='black',lwd=1.0,lty=1,
-                          main=paste0('c',i,' - Dimension:',j),
-                          #ylab=expression(partialdiff * paste(ylab, "/", partialdiff, "t")),
-                          xlab='domain')
-                  # plot motifs with different colors 
-                  col_i <- 1
-                  col <- rainbow(sum(p_clean_i))
-                  lapply(seq_len(K),function(k)
-                   {
-                    if(p_clean_i[k]==1)
-                    {
-                      univariate_motif <- probKMA_results$V0_clean[[k]][,j]
-                      v_dom <- V_dom[[k]]
-                      dom_seq <- seq.int(s_i[k],s_i[k]+max(0,length(v_dom)-1))
-                      if(transformed)
-                      {
-                        max_num = max(univariate_mat[dom_seq[v_dom]], na.rm=TRUE)
-                        min_num = min(univariate_mat[dom_seq[v_dom]], na.rm=TRUE)
-                        univariate_motif[v_dom] = univariate_motif[v_dom]*(max_num - min_num) + min_num
-                      }
-                      lines(dom_seq[v_dom], univariate_motif[v_dom], col = col[col_i] ,lwd=2.5)
-                      rect(dom_seq[v_dom][1],min(univariate_mat,na.rm=TRUE)-10, tail(dom_seq[v_dom], n=1), max(univariate_mat,na.rm=TRUE)+10,
-                                      border = scales::alpha(col[col_i], 0.1), col = scales::alpha(col[col_i], 0.1)) 
-                      col_i <<- col_i + 1
-                    }
-                    })
-                    par(mar=c(0,0,0,0))
-                    plot.new()
-                    legend('left',legend=paste0('motif',which(p_clean_i==1)),col=col,lwd=7,lty=1,bty="n",xpd=TRUE)
-                    })
-        return()},probKMA_results$Y0[ has_a_motif] # consider curves with at least one embedded motif
-                  ,S_clean_i[ has_a_motif],P_clean_i[has_a_motif],seq_len(N)[has_a_motif])
-      mapply(function(v,v_dom,s_k,p_clean_k,k){
-        layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
-        keep=which(p_clean_k==1)
-        Y_inters_k=mapply(
-          function(y,s_k_i,v_dom){
-            v_len=length(v_dom)
-            d=ncol(y)
-            y_len=nrow(y)
-            index=max(1,s_k_i)-1+seq_len(v_len-max(0,1-s_k_i))
-            Y_inters_k=rbind(matrix(NA,nrow=max(0,1-s_k_i),ncol=d),
-                             matrix(y[index[index<=y_len],],ncol=d),
-                             matrix(NA,nrow=sum(index>y_len),ncol=d))
-            return(Y_inters_k)},
-          probKMA_results$Y0[keep],s_k[keep],MoreArgs=list(v_dom),SIMPLIFY=FALSE)
-        Y0_diff_k=lapply(Y_inters_k,
-                         function(Y0_inters_k){
-                           y0_min=apply(Y0_inters_k, 2, min, na.rm = TRUE)
-                           y0_max=apply(Y0_inters_k, 2, max, na.rm = TRUE)
-                           y0_diff=y0_max-y0_min
-                           return(y0_diff)
-                         })
-        lapply(seq_len(d),
-               function(j){
-                 par(mar=c(3,4,4,2)+0.1)
-                 y_plot=matrix(NA,nrow=length(v_dom),ncol=length(Y_inters_k))
-                 if(transformed){
-                   y_plot[v_dom,]=Reduce('cbind',
-                                         mapply(function(Y_inters_k, Y_diff_k) {
-                                           y0_min=min(Y_inters_k[,j])
-                                           y0_norm = t( (t(Y_inters_k[,j]) - y0_min) / Y_diff_k[j] )
-                                           y0_const = (Y_diff_k[j] == 0)
-                                           y0_norm[,y0_const] = 0.5
-                                           return(y0_norm)},
-                                           Y_inters_k, Y0_diff_k, SIMPLIFY=FALSE) ) 
-                 }else{
-                   y_plot[v_dom,]=Reduce('cbind',lapply(Y_inters_k,function(Y_inters_k) Y_inters_k[,j]))
-                 }
-                 
-                 matplot(y_plot,type='l',col=seq_len(N)+1,lwd=1,lty=1,ylab=ylab[j],main=paste('Motif',k,'-',ylab[j]))
-                 points(v[,j],type='l',col='black',lwd=7,lty=1)
-                 par(mar=c(0,0,0,0))
-                 plot.new()
-                 legend('left',legend='motif center',col='black',lwd=7,lty=1,bty="n",xpd=TRUE)
-               })
-        return()},
-        probKMA_results$V0_clean,V_dom,S_clean_k,P_clean_k,seq_len(K))
-    }else
-    {
+      # single line plot
+      
+      # mapply(function(curve_i,s_i,p_clean_i,i){
+      #   layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
+      #   lapply(seq_len(d),
+      #          function(j)
+      #            {
+      #             par(mar=c(3,4,4,2)+0.1)
+      #             univariate_mat <- curve_i[,j]
+      #             matplot(univariate_mat,type="l",col='black',lwd=1.0,lty=1,
+      #                     main=paste0('c',i,' - Dimension:',j),
+      #                     #ylab=expression(partialdiff * paste(ylab, "/", partialdiff, "t")),
+      #                     xlab='domain')
+      #             # plot motifs with different colors 
+      #             col_i <- 1
+      #             col <- rainbow(sum(p_clean_i))
+      #             lapply(seq_len(K),function(k)
+      #              {
+      #               if(p_clean_i[k]==1)
+      #               {
+      #                 univariate_motif <- probKMA_results$V0_clean[[k]][,j]
+      #                 v_dom <- V_dom[[k]]
+      #                 dom_seq <- seq.int(s_i[k],s_i[k]+max(0,length(v_dom)-1))
+      #                 if(transformed)
+      #                 {
+      #                   max_num = max(univariate_mat[dom_seq[v_dom]], na.rm=TRUE)
+      #                   min_num = min(univariate_mat[dom_seq[v_dom]], na.rm=TRUE)
+      #                   univariate_motif[v_dom] = univariate_motif[v_dom]*(max_num - min_num) + min_num
+      #                 }
+      #                 lines(dom_seq[v_dom], univariate_motif[v_dom], col = col[col_i] ,lwd=2.5)
+      #                 rect(dom_seq[v_dom][1],min(univariate_mat,na.rm=TRUE)-10, tail(dom_seq[v_dom], n=1), max(univariate_mat,na.rm=TRUE)+10,
+      #                                 border = scales::alpha(col[col_i], 0.1), col = scales::alpha(col[col_i], 0.1)) 
+      #                 col_i <<- col_i + 1
+      #               }
+      #               })
+      #               par(mar=c(0,0,0,0))
+      #               plot.new()
+      #               legend('left',legend=paste0('motif',which(p_clean_i==1)),col=col,lwd=7,lty=1,bty="n",xpd=TRUE)
+      #               })
+      #   return()},probKMA_results$Y0[ has_a_motif] # consider curves with at least one embedded motif
+      #             ,S_clean_i[ has_a_motif],P_clean_i[has_a_motif],seq_len(N)[has_a_motif])
+      # mapply(function(v,v_dom,s_k,p_clean_k,k){
+      #   layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
+      #   keep=which(p_clean_k==1)
+      #   Y_inters_k=mapply(
+      #     function(y,s_k_i,v_dom){
+      #       v_len=length(v_dom)
+      #       d=ncol(y)
+      #       y_len=nrow(y)
+      #       index=max(1,s_k_i)-1+seq_len(v_len-max(0,1-s_k_i))
+      #       Y_inters_k=rbind(matrix(NA,nrow=max(0,1-s_k_i),ncol=d),
+      #                        matrix(y[index[index<=y_len],],ncol=d),
+      #                        matrix(NA,nrow=sum(index>y_len),ncol=d))
+      #       return(Y_inters_k)},
+      #     probKMA_results$Y0[keep],s_k[keep],MoreArgs=list(v_dom),SIMPLIFY=FALSE)
+      #   Y0_diff_k=lapply(Y_inters_k,
+      #                    function(Y0_inters_k){
+      #                      y0_min=apply(Y0_inters_k, 2, min, na.rm = TRUE)
+      #                      y0_max=apply(Y0_inters_k, 2, max, na.rm = TRUE)
+      #                      y0_diff=y0_max-y0_min
+      #                      return(y0_diff)
+      #                    })
+      #   lapply(seq_len(d),
+      #          function(j){
+      #            par(mar=c(3,4,4,2)+0.1)
+      #            y_plot=matrix(NA,nrow=length(v_dom),ncol=length(Y_inters_k))
+      #            if(transformed){
+      #              y_plot[v_dom,]=Reduce('cbind',
+      #                                    mapply(function(Y_inters_k, Y_diff_k) {
+      #                                      y0_min=min(Y_inters_k[,j])
+      #                                      y0_norm = t( (t(Y_inters_k[,j]) - y0_min) / Y_diff_k[j] )
+      #                                      y0_const = (Y_diff_k[j] == 0)
+      #                                      y0_norm[,y0_const] = 0.5
+      #                                      return(y0_norm)},
+      #                                      Y_inters_k, Y0_diff_k, SIMPLIFY=FALSE) ) 
+        #          }else{
+        #            y_plot[v_dom,]=Reduce('cbind',lapply(Y_inters_k,function(Y_inters_k) Y_inters_k[,j]))
+        #          }
+        #          
+        #          matplot(y_plot,type='l',col=seq_len(N)+1,lwd=1,lty=1,ylab=ylab[j],main=paste('Motif',k,'-',ylab[j]))
+        #          points(v[,j],type='l',col='black',lwd=7,lty=1)
+        #          par(mar=c(0,0,0,0))
+        #          plot.new()
+        #          legend('left',legend='motif center',col='black',lwd=7,lty=1,bty="n",xpd=TRUE)
+        #        })
+        # return()},
+        # probKMA_results$V0_clean,V_dom,S_clean_k,P_clean_k,seq_len(K))
+    }else{
+      print("working on it")
       ##### Plot curves and derivatives all together with the embedded motifs  ############################################################################################
-      mapply(function(v0,v1,v_dom,s_k,p_clean_k,k)
-      {
-        layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
-        keep=which(p_clean_k==1)
-        # matrix extension (eventually)
-        full_mat <- probKMA_results$Y0
-        full_mat_dev <- probKMA_results$Y1
-        # max len of the curves
-        maxLen <-  max(sapply(full_mat, nrow))
-        # Extended full matrix
-        full_mat <- sapply(full_mat, padding, maxLen,simplify = FALSE)
-        full_mat_dev <- sapply(full_mat_dev, padding, maxLen,simplify = FALSE)
-        # Join
-        full_mat <- do.call(cbind, full_mat)
-        full_mat_dev <- do.call(cbind, full_mat_dev)
-        dom_sequence <- sapply(s_k[keep], function(x){seq(x,x+max(0,length(v_dom)-1))},simplify=TRUE)
-        # Extract "keep" submatrix
-        column <- numeric(length(keep) * d)
-        for (i in 1:length(keep)) {
-          start_col <- (keep[i] - 1) * d + 1
-          end_col <- start_col + d - 1
-          column[((i - 1) * d + 1):((i - 1) * d + d)] <- start_col:end_col
-        }
-        # Plot curves all together 
-        lapply(seq_len(d),
-               function(j){
-                 par(mar=c(3,4,4,2)+0.1)
-                 matplot(full_mat[,seq(j,ncol(full_mat),by = d)],col=scales::alpha('gray30',0.30),lwd=1.5,lty=1,type = 'l',
-                         main=paste0('Curves & ','Motif_',k,' - Dimension:',j,'\n',
-                                     'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
-                                     ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))))
-                 univariate_mat <- as.matrix(as.matrix(full_mat[,column])[,seq(j,d*length(keep),by = d)])
-                 matplot(univariate_mat,type="l",col=rainbow(length(keep)),lwd=1.5,lty=5,
-                         main=paste0('Curves & ','Motif_',k,' - Dimension:',j,'\n',
-                                     'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
-                                     ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))),add = TRUE)
-                 for (i in 1:ncol(dom_sequence)) {
-                   lines(dom_sequence[,i],univariate_mat[dom_sequence[,i],i], col = 'red',lwd=2.5)
-                   rect(dom_sequence[1,i], min(univariate_mat,na.rm=TRUE)-10, tail(dom_sequence[,i], n=1), max(univariate_mat,na.rm=TRUE)+10,
-                        border = scales::alpha(rainbow(length(keep))[i], 0.05), col = scales::alpha(rainbow(length(keep))[i], 0.05))
-                 }
-                 title(sub = paste('Curves & ', 'Motif_', k, ' - Dimension:', j))
-                 par(mar=c(0,0,0,0))
-                 plot.new()
-                 legend("topright", legend = c(paste0("c",(1:N)[keep]),paste0('motif_',k)),
-                        col = c(rainbow(length(keep)),'red'), lty = 1,lwd=c(rep(1,length(keep)),4),cex=1.5)
-               })
-        # Plot Derivatives all together 
-        lapply(seq_len(d),
-               function(j){
-                 par(mar=c(3,4,4,2)+0.1)
-                 matplot(full_mat_dev[,seq(j,ncol(full_mat_dev),by = d)],col=scales::alpha('gray30',0.30),lwd=1.5,lty=1,type = 'l',
-                         main=paste0('Derivatives & ','Motif_',k,' - Dimension:',j,'\n',
-                                     'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
-                                     ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))))
-                 univariate_mat <- as.matrix(as.matrix(full_mat_dev[,column])[,seq(j,length(keep),by = d)])
-                 matplot(univariate_mat,type="l",col=rainbow(length(keep)),lwd=1.5,lty=5,
-                         main=paste0('Derivatives & ','Motif_',k,' - Dimension:',j,'\n',
-                                     'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
-                                     ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))),add = TRUE)
-                 for (i in 1:ncol(dom_sequence)) {
-                   lines(dom_sequence[,i],univariate_mat[dom_sequence[,i],i], col = 'red',lwd=2.5)
-                   rect(dom_sequence[1,i], min(univariate_mat,na.rm=TRUE)-10, tail(dom_sequence[,i], n=1), max(univariate_mat,na.rm=TRUE)+10,
-                        border = scales::alpha(rainbow(length(keep))[i], 0.05), col = scales::alpha(rainbow(length(keep))[i], 0.05))
-                 }
-                 title(sub = paste('Derivatives & ', 'Motif:', k, ' - Dimension:', j))
-                 par(mar=c(0,0,0,0))
-                 plot.new()
-                 legend("topright", legend = c(paste0("c",(1:N)[keep]),paste0('motif_',k)),
-                        col = c(rainbow(length(keep)),'red'), lty = 1,lwd=c(rep(1,length(keep)),4),cex=1.5)
-               })
-        
-        return()},probKMA_results$V0_clean,probKMA_results$V1_clean,
-                  V_dom,S_clean_k,P_clean_k,seq_len(K))
+      # colorful plot with everything together
+      
+      # mapply(function(v0,v1,v_dom,s_k,p_clean_k,k)
+      # {
+      #   layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
+      #   keep=which(p_clean_k==1)
+      #   # matrix extension (eventually)
+      #   full_mat <- probKMA_results$Y0
+      #   full_mat_dev <- probKMA_results$Y1
+      #   # max len of the curves
+      #   maxLen <-  max(sapply(full_mat, nrow))
+      #   # Extended full matrix
+      #   full_mat <- sapply(full_mat, padding, maxLen,simplify = FALSE)
+      #   full_mat_dev <- sapply(full_mat_dev, padding, maxLen,simplify = FALSE)
+      #   # Join
+      #   full_mat <- do.call(cbind, full_mat)
+      #   full_mat_dev <- do.call(cbind, full_mat_dev)
+      #   dom_sequence <- sapply(s_k[keep], function(x){seq(x,x+max(0,length(v_dom)-1))},simplify=TRUE)
+      #   # Extract "keep" submatrix
+      #   column <- numeric(length(keep) * d)
+      #   for (i in 1:length(keep)) {
+      #     start_col <- (keep[i] - 1) * d + 1
+      #     end_col <- start_col + d - 1
+      #     column[((i - 1) * d + 1):((i - 1) * d + d)] <- start_col:end_col
+      #   }
+      #   # Plot curves all together 
+      #   lapply(seq_len(d),
+      #          function(j){
+      #            par(mar=c(3,4,4,2)+0.1)
+      #            matplot(full_mat[,seq(j,ncol(full_mat),by = d)],col=scales::alpha('gray30',0.30),lwd=1.5,lty=1,type = 'l',
+      #                    main=paste0('Curves & ','Motif_',k,' - Dimension:',j,'\n',
+      #                                'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
+      #                                ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))))
+      #            univariate_mat <- as.matrix(as.matrix(full_mat[,column])[,seq(j,d*length(keep),by = d)])
+      #            matplot(univariate_mat,type="l",col=rainbow(length(keep)),lwd=1.5,lty=5,
+      #                    main=paste0('Curves & ','Motif_',k,' - Dimension:',j,'\n',
+      #                                'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
+      #                                ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))),add = TRUE)
+      #            for (i in 1:ncol(dom_sequence)) {
+      #              lines(dom_sequence[,i],univariate_mat[dom_sequence[,i],i], col = 'red',lwd=2.5)
+      #              rect(dom_sequence[1,i], min(univariate_mat,na.rm=TRUE)-10, tail(dom_sequence[,i], n=1), max(univariate_mat,na.rm=TRUE)+10,
+      #                   border = scales::alpha(rainbow(length(keep))[i], 0.05), col = scales::alpha(rainbow(length(keep))[i], 0.05))
+      #            }
+      #            title(sub = paste('Curves & ', 'Motif_', k, ' - Dimension:', j))
+      #            par(mar=c(0,0,0,0))
+      #            plot.new()
+      #            legend("topright", legend = c(paste0("c",(1:N)[keep]),paste0('motif_',k)),
+      #                   col = c(rainbow(length(keep)),'red'), lty = 1,lwd=c(rep(1,length(keep)),4),cex=1.5)
+      #          })
+      #   # Plot Derivatives all together 
+      #   lapply(seq_len(d),
+      #          function(j){
+      #            par(mar=c(3,4,4,2)+0.1)
+      #            matplot(full_mat_dev[,seq(j,ncol(full_mat_dev),by = d)],col=scales::alpha('gray30',0.30),lwd=1.5,lty=1,type = 'l',
+      #                    main=paste0('Derivatives & ','Motif_',k,' - Dimension:',j,'\n',
+      #                                'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
+      #                                ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))))
+      #            univariate_mat <- as.matrix(as.matrix(full_mat_dev[,column])[,seq(j,length(keep),by = d)])
+      #            matplot(univariate_mat,type="l",col=rainbow(length(keep)),lwd=1.5,lty=5,
+      #                    main=paste0('Derivatives & ','Motif_',k,' - Dimension:',j,'\n',
+      #                                'Number of instances: ',ncol(dom_sequence),' - sil_avg: ',
+      #                                ifelse(is.null(sil_avg),"",round(sil_avg[k], digits = 3))),add = TRUE)
+      #            for (i in 1:ncol(dom_sequence)) {
+      #              lines(dom_sequence[,i],univariate_mat[dom_sequence[,i],i], col = 'red',lwd=2.5)
+      #              rect(dom_sequence[1,i], min(univariate_mat,na.rm=TRUE)-10, tail(dom_sequence[,i], n=1), max(univariate_mat,na.rm=TRUE)+10,
+      #                   border = scales::alpha(rainbow(length(keep))[i], 0.05), col = scales::alpha(rainbow(length(keep))[i], 0.05))
+      #            }
+      #            title(sub = paste('Derivatives & ', 'Motif:', k, ' - Dimension:', j))
+      #            par(mar=c(0,0,0,0))
+      #            plot.new()
+      #            legend("topright", legend = c(paste0("c",(1:N)[keep]),paste0('motif_',k)),
+      #                   col = c(rainbow(length(keep)),'red'), lty = 1,lwd=c(rep(1,length(keep)),4),cex=1.5)
+      #          })
+      #   
+      #   return()},probKMA_results$V0_clean,probKMA_results$V1_clean,
+      #             V_dom,S_clean_k,P_clean_k,seq_len(K))
       
     ##### for each curve plot the embedded motifs  ############################################################################################
-    mapply(function(curve_i,dev_i,s_i,p_clean_i,i){
-      #plot curve with embedded motif 
-      layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
-      lapply(seq_len(d),
-             function(j)
-             {
-               par(mar=c(3,4,4,2)+0.1)
-               univariate_mat <- curve_i[,j]
-               matplot(univariate_mat,type="l",col='black',lwd=1.0,lty=1,
-                       main=paste0('c',i,' - Dimension:',j),
-                       #ylab=expression(partialdiff * paste(ylab, "/", partialdiff, "t")),
-                       xlab='domain')
-               # plot motifs with different colors 
-               col_i <- 1
-               col <- rainbow(sum(p_clean_i))
-               lapply(seq_len(K),function(k)
-               {
-                 if(p_clean_i[k]==1)
-                 {
-                   univariate_motif <- probKMA_results$V0_clean[[k]][,j]
-                   v_dom <- V_dom[[k]]
-                   dom_seq <- seq.int(s_i[k],s_i[k]+max(0,length(v_dom)-1))
-                   if(transformed)
-                   {
-                     max_num = max(univariate_mat[dom_seq[v_dom]], na.rm=TRUE)
-                     min_num = min(univariate_mat[dom_seq[v_dom]], na.rm=TRUE)
-                     univariate_motif[v_dom] = univariate_motif[v_dom]*(max_num - min_num) + min_num
-                   }
-                   lines(dom_seq[v_dom], univariate_motif[v_dom], col = col[col_i] ,lwd=2.5)
-                   rect(dom_seq[v_dom][1],min(univariate_mat,na.rm=TRUE)-10, tail(dom_seq[v_dom], n=1), max(univariate_mat,na.rm=TRUE)+10,
-                        border = scales::alpha(col[col_i], 0.1), col = scales::alpha(col[col_i], 0.1)) 
-                   col_i <<- col_i + 1
-                 }
-               })
-               par(mar=c(0,0,0,0))
-               plot.new()
-               legend('left',legend=paste0('motif_',which(p_clean_i==1)),col=col,lwd=7,lty=1,bty="n",xpd=TRUE)
-             })
+    # for each curve the motif highlighted
+      
+      # mapply(function(curve_i,dev_i,s_i,p_clean_i,i){
+      # #plot curve with embedded motif 
+      # layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
+      # lapply(seq_len(d),
+      #        function(j)
+      #        {
+      #          par(mar=c(3,4,4,2)+0.1)
+      #          univariate_mat <- curve_i[,j]
+      #          matplot(univariate_mat,type="l",col='black',lwd=1.0,lty=1,
+      #                  main=paste0('c',i,' - Dimension:',j),
+      #                  #ylab=expression(partialdiff * paste(ylab, "/", partialdiff, "t")),
+      #                  xlab='domain')
+      #          # plot motifs with different colors 
+      #          col_i <- 1
+      #          col <- rainbow(sum(p_clean_i))
+      #          lapply(seq_len(K),function(k)
+      #          {
+      #            if(p_clean_i[k]==1)
+      #            {
+      #              univariate_motif <- probKMA_results$V0_clean[[k]][,j]
+      #              v_dom <- V_dom[[k]]
+      #              dom_seq <- seq.int(s_i[k],s_i[k]+max(0,length(v_dom)-1))
+      #              if(transformed)
+      #              {
+      #                max_num = max(univariate_mat[dom_seq[v_dom]], na.rm=TRUE)
+      #                min_num = min(univariate_mat[dom_seq[v_dom]], na.rm=TRUE)
+      #                univariate_motif[v_dom] = univariate_motif[v_dom]*(max_num - min_num) + min_num
+      #              }
+      #              lines(dom_seq[v_dom], univariate_motif[v_dom], col = col[col_i] ,lwd=2.5)
+      #              rect(dom_seq[v_dom][1],min(univariate_mat,na.rm=TRUE)-10, tail(dom_seq[v_dom], n=1), max(univariate_mat,na.rm=TRUE)+10,
+      #                   border = scales::alpha(col[col_i], 0.1), col = scales::alpha(col[col_i], 0.1)) 
+      #              col_i <<- col_i + 1
+      #            }
+      #          })
+      #          par(mar=c(0,0,0,0))
+      #          plot.new()
+      #          legend('left',legend=paste0('motif_',which(p_clean_i==1)),col=col,lwd=7,lty=1,bty="n",xpd=TRUE)
+      #        })
       
       #plot derivative with embedded motif 
-      layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
-      lapply(seq_len(d),
-             function(j)
-             {
-               par(mar=c(3,4,4,2)+0.1)
-               univariate_mat_curve <- curve_i[,j]
-               univariate_mat <- dev_i[,j]
-               matplot(univariate_mat,type="l",col='black',lwd=1.0,lty=1,
-                       main=paste0('c',i,' - Dimension:',j),
-                       ylab=expression(partialdiff * paste(ylab, "/", partialdiff, "t")),
-                       xlab='domain')
-               # plot motifs with different colors 
-               col_i <- 1
-               col <- rainbow(sum(p_clean_i))
-               lapply(seq_len(K),function(k)
-               {
-                 if(p_clean_i[k]==1)
-                 {
-                   univariate_motif <- probKMA_results$V1_clean[[k]][,j]
-                   v_dom <- V_dom[[k]]
-                   dom_seq <- seq.int(s_i[k],s_i[k]+max(0,length(v_dom)-1))
-                   if(transformed)
-                   {
-                     max_num = max(univariate_mat_curve[dom_seq[v_dom]], na.rm=TRUE)
-                     min_num = min(univariate_mat_curve[dom_seq[v_dom]], na.rm=TRUE)
-                     univariate_motif[v_dom] = univariate_motif[v_dom]*(max_num - min_num)
-                   }
-                   lines(dom_seq[v_dom], univariate_motif[v_dom], col = col[col_i] ,lwd=2.5)
-                   rect(dom_seq[v_dom][1],min(univariate_mat,na.rm=TRUE)-10, tail(dom_seq[v_dom], n=1), max(univariate_mat,na.rm=TRUE)+10,
-                        border = scales::alpha(col[col_i], 0.1), col = scales::alpha(col[col_i], 0.1)) 
-                   col_i <<- col_i + 1
-                 }
-               })
-               par(mar=c(0,0,0,0))
-               plot.new()
-               legend('left',legend=paste0('motif_',which(p_clean_i==1)),col=col,lwd=7,lty=1,bty="n",xpd=TRUE)
-             })
+      #for each derivative curve the motif highlighted
       
-      return()},probKMA_results$Y0[ has_a_motif], # consider curves with at least one embedded motif
-                probKMA_results$Y1[ has_a_motif],
-                S_clean_i[ has_a_motif],P_clean_i[has_a_motif],seq_len(N)[has_a_motif])
+      # layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
+      # lapply(seq_len(d),
+      #        function(j)
+      #        {
+      #          par(mar=c(3,4,4,2)+0.1)
+      #          univariate_mat_curve <- curve_i[,j]
+      #          univariate_mat <- dev_i[,j]
+      #          matplot(univariate_mat,type="l",col='black',lwd=1.0,lty=1,
+      #                  main=paste0('c',i,' - Dimension:',j),
+      #                  ylab=expression(partialdiff * paste(ylab, "/", partialdiff, "t")),
+      #                  xlab='domain')
+      #          # plot motifs with different colors 
+      #          col_i <- 1
+      #          col <- rainbow(sum(p_clean_i))
+      #          lapply(seq_len(K),function(k)
+      #          {
+      #            if(p_clean_i[k]==1)
+      #            {
+      #              univariate_motif <- probKMA_results$V1_clean[[k]][,j]
+      #              v_dom <- V_dom[[k]]
+      #              dom_seq <- seq.int(s_i[k],s_i[k]+max(0,length(v_dom)-1))
+      #              if(transformed)
+      #              {
+      #                max_num = max(univariate_mat_curve[dom_seq[v_dom]], na.rm=TRUE)
+      #                min_num = min(univariate_mat_curve[dom_seq[v_dom]], na.rm=TRUE)
+      #                univariate_motif[v_dom] = univariate_motif[v_dom]*(max_num - min_num)
+      #              }
+      #              lines(dom_seq[v_dom], univariate_motif[v_dom], col = col[col_i] ,lwd=2.5)
+      #              rect(dom_seq[v_dom][1],min(univariate_mat,na.rm=TRUE)-10, tail(dom_seq[v_dom], n=1), max(univariate_mat,na.rm=TRUE)+10,
+      #                   border = scales::alpha(col[col_i], 0.1), col = scales::alpha(col[col_i], 0.1)) 
+      #              col_i <<- col_i + 1
+      #            }
+      #          })
+      #          par(mar=c(0,0,0,0))
+      #          plot.new()
+      #          legend('left',legend=paste0('motif_',which(p_clean_i==1)),col=col,lwd=7,lty=1,bty="n",xpd=TRUE)
+      #        })
+      # 
+      # return()},probKMA_results$Y0[ has_a_motif], # consider curves with at least one embedded motif
+      #           probKMA_results$Y1[ has_a_motif],
+      #           S_clean_i[ has_a_motif],P_clean_i[has_a_motif],seq_len(N)[has_a_motif])
     
       mapply(function(v0,v1,v_dom,s_k,p_clean_k,k){
         keep=which(p_clean_k==1)
@@ -580,7 +590,7 @@ probKMA_plot <- function(probKMA_results,plot,ylab='',sil_avg=NULL,cleaned=FALSE
     }
   }
   
-  layout(matrix(1:(2*d),ncol=2,byrow=TRUE),widths=c(8.5,1))
+  layout(matrix(1:(2*d), ncol=2, byrow=TRUE), widths=c(8.5,1))
   ### plot motifs ############################################################################################
   if(cleaned){
     lapply(seq_len(d),
